@@ -1,8 +1,10 @@
 from .chisquare import ChiSquareDrift
 from .classifier import ClassifierDrift
 from .ks import KSDrift
+from .learned_kernel import LearnedKernelDrift
 from .lsdd import LSDDDrift
 from .lsdd_online import LSDDDriftOnline
+from .spot_the_diff import SpotTheDiffDrift
 from .mmd import MMDDrift
 from .mmd_online import MMDDriftOnline
 from .model_uncertainty import ClassifierUncertaintyDrift, RegressorUncertaintyDrift
@@ -12,11 +14,13 @@ __all__ = [
     "ChiSquareDrift",
     "ClassifierDrift",
     "KSDrift",
+    "LearnedKernelDrift",
     "LSDDDrift",
     "LSDDDriftOnline",
     "MMDDrift",
     "MMDDriftOnline",
     "TabularDrift",
     "ClassifierUncertaintyDrift",
-    "RegressorUncertaintyDrift"
+    "RegressorUncertaintyDrift",
+    "SpotTheDiffDrift"
 ]
