@@ -24,8 +24,10 @@ class ClassifierDrift:
             preprocess_fn: Optional[Callable] = None,
             preds_type: str = 'probs',
             binarize_preds: bool = False,
+            reg_loss_fn: Callable = (lambda model: 0),
             train_size: Optional[float] = .75,
             n_folds: Optional[int] = None,
+            retrain_from_scratch: bool = True,
             seed: int = 0,
             optimizer: Optional[Callable] = None,
             learning_rate: float = 1e-3,
@@ -67,6 +69,8 @@ class ClassifierDrift:
         binarize_preds
             Whether to test for discrepency on soft  (e.g. probs/logits) model predictions directly
             with a K-S test or binarise to 0-1 prediction errors and apply a binomial test.
+        reg_loss_fn
+            The regularisation term reg_loss_fn(model) is added to the loss function being optimized.
         train_size
             Optional fraction (float between 0 and 1) of the dataset used to train the classifier.
             The drift is detected on `1 - train_size`. Cannot be used in combination with `n_folds`.
@@ -75,6 +79,9 @@ class ClassifierDrift:
             on all the out-of-fold instances. This allows to leverage all the reference and test data
             for drift detection at the expense of longer computation. If both `train_size` and `n_folds`
             are specified, `n_folds` is prioritized.
+        retrain_from_scratch
+            Whether the classifier should be retrained from scratch for each set of test data or whether
+            it should instead continue training from where it left off on the previous set.
         seed
             Optional random seed for fold selection.
         optimizer
@@ -125,7 +132,6 @@ class ClassifierDrift:
                 kwargs.update({'dataset': TFDataset})
             self._detector = ClassifierDriftTF(*args, **kwargs)  # type: ignore
         else:
-            kwargs.pop('compile_kwargs', None)
             if dataset is None:
                 kwargs.update({'dataset': TorchDataset})
             if dataloader is None:
@@ -134,8 +140,8 @@ class ClassifierDrift:
         self.meta = self._detector.meta
 
     def predict(self, x: Union[np.ndarray, list],  return_p_val: bool = True,
-                return_distance: bool = True, return_probs: bool = True) \
-            -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
+                return_distance: bool = True, return_probs: bool = True, return_model: bool = True) \
+            -> Dict[str, Dict[str, Union[str, int, float, Callable]]]:
         """
         Predict whether a batch of data has drifted from the reference data.
 
@@ -151,13 +157,15 @@ class ClassifierDrift:
         return_probs
             Whether to return the instance level classifier probabilities for the reference and test data
             (0=reference data, 1=test data).
+        return_model
+            Whether to return the updated model trained to discriminate reference and test instances.
 
         Returns
         -------
         Dictionary containing 'meta' and 'data' dictionaries.
         'meta' has the model's metadata.
-        'data' contains the drift prediction and optionally the performance of the classifier
+        'data' contains the drift prediction and optionally the p-value, performance of the classifier
         relative to its expectation under the no-change null, the out-of-fold classifier model
-        prediction probabilities on the reference and test data.
+        prediction probabilities on the reference and test data, and the trained model.
         """
-        return self._detector.predict(x, return_p_val, return_distance, return_probs)
+        return self._detector.predict(x, return_p_val, return_distance, return_probs, return_model)
