@@ -12,7 +12,7 @@ if has_tensorflow:
 class LSDDDriftOnline:
     def __init__(
             self,
-            x_ref: np.ndarray,
+            x_ref: Union[np.ndarray, list],
             ert: float,
             window_size: int,
             backend: str = 'tensorflow',
@@ -107,7 +107,7 @@ class LSDDDriftOnline:
         "Resets the detector but does not reconfigure thresholds."
         self._detector.reset()
 
-    def predict(self, x_t: np.ndarray, return_test_stat: bool = True) \
+    def predict(self, x_t: Union[np.ndarray, list], return_test_stat: bool = True) \
             -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
         """
         Predict whether the most recent window of data has drifted from the reference data.
