@@ -22,8 +22,10 @@ class ClassifierDriftTF(BaseClassifierDrift):
             preprocess_fn: Optional[Callable] = None,
             preds_type: str = 'preds',
             binarize_preds: bool = False,
+            reg_loss_fn: Callable = (lambda model: 0),
             train_size: Optional[float] = .75,
             n_folds: Optional[int] = None,
+            retrain_from_scratch: bool = True,
             seed: int = 0,
             optimizer: tf.keras.optimizers = tf.keras.optimizers.Adam,
             learning_rate: float = 1e-3,
@@ -61,6 +63,8 @@ class ClassifierDriftTF(BaseClassifierDrift):
         binarize_preds
             Whether to test for discrepency on soft (e.g. prob/log-prob) model predictions directly
             with a K-S test or binarise to 0-1 prediction errors and apply a binomial test.
+        reg_loss_fn
+            The regularisation term reg_loss_fn(model) is added to the loss function being optimized.
         train_size
             Optional fraction (float between 0 and 1) of the dataset used to train the classifier.
             The drift is detected on `1 - train_size`. Cannot be used in combination with `n_folds`.
@@ -69,6 +73,9 @@ class ClassifierDriftTF(BaseClassifierDrift):
             on all the out-of-fold predictions. This allows to leverage all the reference and test data
             for drift detection at the expense of longer computation. If both `train_size` and `n_folds`
             are specified, `n_folds` is prioritized.
+        retrain_from_scratch
+            Whether the classifier should be retrained from scratch for each set of test data or whether
+            it should instead continue training from where it left off on the previous set.
         seed
             Optional random seed for fold selection.
         optimizer
@@ -99,18 +106,20 @@ class ClassifierDriftTF(BaseClassifierDrift):
             binarize_preds=binarize_preds,
             train_size=train_size,
             n_folds=n_folds,
+            retrain_from_scratch=retrain_from_scratch,
             seed=seed,
             data_type=data_type
         )
         self.meta.update({'backend': 'tensorflow'})
 
         # define and compile classifier model
-        self.model = model
+        self.original_model = model
+        self.model = clone_model(model)
         self.loss_fn = BinaryCrossentropy(from_logits=(self.preds_type == 'logits'))
         self.dataset = partial(dataset, batch_size=batch_size, shuffle=True)
         self.predict_fn = partial(predict_batch, preprocess_fn=preprocess_batch_fn, batch_size=batch_size)
         self.train_kwargs = {'optimizer': optimizer(learning_rate=learning_rate), 'epochs': epochs,
-                             'preprocess_fn': preprocess_batch_fn, 'verbose': verbose}
+                             'reg_loss_fn': reg_loss_fn, 'preprocess_fn': preprocess_batch_fn, 'verbose': verbose}
         if isinstance(train_kwargs, dict):
             self.train_kwargs.update(train_kwargs)
 
@@ -145,11 +154,12 @@ class ClassifierDriftTF(BaseClassifierDrift):
             else:
                 raise TypeError(f'x needs to be of type np.ndarray or list and not {type(x)}.')
             ds_tr = self.dataset(x_tr, y_tr)
-            model = clone_model(self.model)
-            train_args = [model, self.loss_fn, None]
+            self.model = clone_model(self.original_model) if self.retrain_from_scratch \
+                else self.model
+            train_args = [self.model, self.loss_fn, None]
             self.train_kwargs.update({'dataset': ds_tr})
             trainer(*train_args, **self.train_kwargs)  # type: ignore
-            preds = self.predict_fn(x_te, model)
+            preds = self.predict_fn(x_te, self.model)
             preds_oof_list.append(preds)
             idx_oof_list.append(idx_te)
         preds_oof = np.concatenate(preds_oof_list, axis=0)
