@@ -1,11 +1,13 @@
-import cloudpickle as cp
 import logging
 import os
-import pickle
+from pathlib import Path
+import dill
 import tensorflow as tf
 from tensorflow.python.keras import backend
 from typing import Tuple, Union
-from urllib.request import urlopen
+from io import BytesIO
+import requests
+from requests import RequestException
 from alibi_detect.base import BaseDetector
 from alibi_detect.ad import AdversarialAE, ModelDistillation
 from alibi_detect.models.tensorflow import PixelCNN
@@ -13,6 +15,9 @@ from alibi_detect.od import (IForest, LLR, Mahalanobis, OutlierAE, OutlierAEGMM,
                              OutlierSeq2Seq, OutlierVAE, OutlierVAEGMM, SpectralResidual)
 from alibi_detect.utils.saving import load_detector  # type: ignore
 
+# do not extend pickle dispatch table so as not to change pickle behaviour
+dill.extend(use_dill=False)
+
 logger = logging.getLogger(__name__)
 
 Data = Union[
@@ -69,7 +74,7 @@ def fetch_tf_model(dataset: str, model: str) -> tf.keras.Model:
     """
     url = 'https://storage.googleapis.com/seldon-models/alibi-detect/classifier/'
     path_model = os.path.join(url, dataset, model, 'model.h5')
-    save_path = tf.keras.utils.get_file(model, path_model)
+    save_path = tf.keras.utils.get_file(Path(model + '.h5').resolve(), path_model)
     if dataset == 'cifar10' and model == 'resnet56':
         custom_objects = {'backend': backend}
     else:
@@ -78,7 +83,7 @@ def fetch_tf_model(dataset: str, model: str) -> tf.keras.Model:
     return clf
 
 
-def fetch_enc_dec(url: str, filepath: str) -> None:
+def fetch_enc_dec(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download encoder and decoder networks.
 
@@ -90,21 +95,21 @@ def fetch_enc_dec(url: str, filepath: str) -> None:
         Local directory to save detector to.
     """
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
-    if not os.path.isdir(model_path):
-        os.mkdir(model_path)
+    model_path = Path(filepath).joinpath('model').resolve()
+    if not model_path.is_dir():
+        model_path.mkdir(parents=True, exist_ok=True)
     # encoder and decoder
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'encoder_net.h5'),
+        model_path.joinpath('encoder_net.h5'),
         os.path.join(url_models, 'encoder_net.h5')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'decoder_net.h5'),
+        model_path.joinpath('decoder_net.h5'),
         os.path.join(url_models, 'decoder_net.h5')
     )
 
 
-def fetch_ae(url: str, filepath: str) -> None:
+def fetch_ae(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download AE outlier detector.
 
@@ -117,23 +122,23 @@ def fetch_ae(url: str, filepath: str) -> None:
     """
     fetch_enc_dec(url, filepath)
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
+    model_path = Path(filepath).joinpath('model').resolve()
     # encoder and decoder
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'checkpoint'),
+        model_path.joinpath('checkpoint'),
         os.path.join(url_models, 'checkpoint')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'ae.ckpt.index'),
+        model_path.joinpath('ae.ckpt.index'),
         os.path.join(url_models, 'ae.ckpt.index')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'ae.ckpt.data-00000-of-00001'),
+        model_path.joinpath('ae.ckpt.data-00000-of-00001'),
         os.path.join(url_models, 'ae.ckpt.data-00000-of-00001')
     )
 
 
-def fetch_ad_ae(url: str, filepath: str, state_dict: dict) -> None:
+def fetch_ad_ae(url: str, filepath: Union[str, os.PathLike], state_dict: dict) -> None:
     """
     Download AE adversarial detector.
 
@@ -148,25 +153,25 @@ def fetch_ad_ae(url: str, filepath: str, state_dict: dict) -> None:
     """
     fetch_enc_dec(url, filepath)
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
+    model_path = Path(filepath).joinpath('model').resolve()
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'model.h5'),
+        model_path.joinpath('model.h5'),
         os.path.join(url_models, 'model.h5')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'checkpoint'),
+        model_path.joinpath('checkpoint'),
         os.path.join(url_models, 'checkpoint')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'ae.ckpt.index'),
+        model_path.joinpath('ae.ckpt.index'),
         os.path.join(url_models, 'ae.ckpt.index')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'ae.ckpt.data-00000-of-00002'),
+        model_path.joinpath('ae.ckpt.data-00000-of-00002'),
         os.path.join(url_models, 'ae.ckpt.data-00000-of-00002')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'ae.ckpt.data-00001-of-00002'),
+        model_path.joinpath('ae.ckpt.data-00001-of-00002'),
         os.path.join(url_models, 'ae.ckpt.data-00001-of-00002')
     )
     hidden_layer_kld = state_dict['hidden_layer_kld']
@@ -174,20 +179,20 @@ def fetch_ad_ae(url: str, filepath: str, state_dict: dict) -> None:
         for i, (_, _) in enumerate(hidden_layer_kld.items()):
             hl = 'model_hl_' + str(i)
             tf.keras.utils.get_file(
-                os.path.join(model_path, hl + '.ckpt.index'),
+                model_path.joinpath(hl + '.ckpt.index'),
                 os.path.join(url_models, hl + '.ckpt.index')
             )
             tf.keras.utils.get_file(
-                os.path.join(model_path, hl + '.ckpt.data-00000-of-00002'),
+                model_path.joinpath(hl + '.ckpt.data-00000-of-00002'),
                 os.path.join(url_models, hl + '.ckpt.data-00000-of-00002')
             )
             tf.keras.utils.get_file(
-                os.path.join(model_path, hl + '.ckpt.data-00001-of-00002'),
+                model_path.joinpath(hl + '.ckpt.data-00001-of-00002'),
                 os.path.join(url_models, hl + '.ckpt.data-00001-of-00002')
             )
 
 
-def fetch_ad_md(url: str, filepath: str) -> None:
+def fetch_ad_md(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download model and distilled model.
 
@@ -199,21 +204,21 @@ def fetch_ad_md(url: str, filepath: str) -> None:
         Local directory to save detector to.
     """
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
-    if not os.path.isdir(model_path):
-        os.mkdir(model_path)
+    model_path = Path(filepath).joinpath('model').resolve()
+    if not model_path.is_dir():
+        model_path.mkdir(parents=True, exist_ok=True)
     # encoder and decoder
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'model.h5'),
+        model_path.joinpath('model.h5'),
         os.path.join(url_models, 'model.h5')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'distilled_model.h5'),
+        model_path.joinpath('distilled_model.h5'),
         os.path.join(url_models, 'distilled_model.h5')
     )
 
 
-def fetch_aegmm(url: str, filepath: str) -> None:
+def fetch_aegmm(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download AEGMM outlier detector.
 
@@ -228,26 +233,26 @@ def fetch_aegmm(url: str, filepath: str) -> None:
     fetch_enc_dec(url, filepath)
     # save GMM network
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
+    model_path = Path(filepath).joinpath('model').resolve()
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'gmm_density_net.h5'),
+        model_path.joinpath('gmm_density_net.h5'),
         os.path.join(url_models, 'gmm_density_net.h5')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'checkpoint'),
+        model_path.joinpath('checkpoint'),
         os.path.join(url_models, 'checkpoint')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'aegmm.ckpt.index'),
+        model_path.joinpath('aegmm.ckpt.index'),
         os.path.join(url_models, 'aegmm.ckpt.index')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'aegmm.ckpt.data-00000-of-00001'),
+        model_path.joinpath('aegmm.ckpt.data-00000-of-00001'),
         os.path.join(url_models, 'aegmm.ckpt.data-00000-of-00001')
     )
 
 
-def fetch_vae(url: str, filepath: str) -> None:
+def fetch_vae(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download VAE outlier detector.
 
@@ -261,22 +266,22 @@ def fetch_vae(url: str, filepath: str) -> None:
     fetch_enc_dec(url, filepath)
     # save VAE weights
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
+    model_path = Path(filepath).joinpath('model').resolve()
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'checkpoint'),
+        model_path.joinpath('checkpoint'),
         os.path.join(url_models, 'checkpoint')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'vae.ckpt.index'),
+        model_path.joinpath('vae.ckpt.index'),
         os.path.join(url_models, 'vae.ckpt.index')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'vae.ckpt.data-00000-of-00001'),
+        model_path.joinpath('vae.ckpt.data-00000-of-00001'),
         os.path.join(url_models, 'vae.ckpt.data-00000-of-00001')
     )
 
 
-def fetch_vaegmm(url: str, filepath: str) -> None:
+def fetch_vaegmm(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download VAEGMM outlier detector.
 
@@ -291,26 +296,26 @@ def fetch_vaegmm(url: str, filepath: str) -> None:
     fetch_enc_dec(url, filepath)
     # save GMM network
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
+    model_path = Path(filepath).joinpath('model').resolve()
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'gmm_density_net.h5'),
+        model_path.joinpath('gmm_density_net.h5'),
         os.path.join(url_models, 'gmm_density_net.h5')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'checkpoint'),
+        model_path.joinpath('checkpoint'),
         os.path.join(url_models, 'checkpoint')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'vaegmm.ckpt.index'),
+        model_path.joinpath('vaegmm.ckpt.index'),
         os.path.join(url_models, 'vaegmm.ckpt.index')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'vaegmm.ckpt.data-00000-of-00001'),
+        model_path.joinpath('vaegmm.ckpt.data-00000-of-00001'),
         os.path.join(url_models, 'vaegmm.ckpt.data-00000-of-00001')
     )
 
 
-def fetch_seq2seq(url: str, filepath: str) -> None:
+def fetch_seq2seq(url: str, filepath: Union[str, os.PathLike]) -> None:
     """
     Download sequence-to-sequence outlier detector.
 
@@ -322,30 +327,30 @@ def fetch_seq2seq(url: str, filepath: str) -> None:
         Local directory to save detector to.
     """
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
-    if not os.path.isdir(model_path):
-        os.mkdir(model_path)
+    model_path = Path(filepath).joinpath('model').resolve()
+    if not model_path.is_dir():
+        model_path.mkdir(parents=True, exist_ok=True)
     # save seq2seq
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'checkpoint'),
+        model_path.joinpath('checkpoint'),
         os.path.join(url_models, 'checkpoint')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'seq2seq.ckpt.index'),
+        model_path.joinpath('seq2seq.ckpt.index'),
         os.path.join(url_models, 'seq2seq.ckpt.index')
     )
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'seq2seq.ckpt.data-00000-of-00001'),
+        model_path.joinpath('seq2seq.ckpt.data-00000-of-00001'),
         os.path.join(url_models, 'seq2seq.ckpt.data-00000-of-00001')
     )
     # save threshold network
     tf.keras.utils.get_file(
-        os.path.join(model_path, 'threshold_net.h5'),
+        model_path.joinpath('threshold_net.h5'),
         os.path.join(url_models, 'threshold_net.h5')
     )
 
 
-def fetch_llr(url: str, filepath: str) -> str:
+def fetch_llr(url: str, filepath: Union[str, os.PathLike]) -> str:
     """
     Download Likelihood Ratio outlier detector.
 
@@ -357,33 +362,34 @@ def fetch_llr(url: str, filepath: str) -> str:
         Local directory to save detector to.
     """
     url_models = os.path.join(url, 'model')
-    model_path = os.path.join(filepath, 'model')
-    if not os.path.isdir(model_path):
-        os.mkdir(model_path)
+    model_path = Path(filepath).joinpath('model').resolve()
+    if not model_path.is_dir():
+        model_path.mkdir(parents=True, exist_ok=True)
     try:
         tf.keras.utils.get_file(
-            os.path.join(model_path, 'model_s.h5'),
+            model_path.joinpath('model_s.h5'),
             os.path.join(url_models, 'model_s.h5')
         )
         tf.keras.utils.get_file(
-            os.path.join(model_path, 'model_b.h5'),
+            model_path.joinpath('model_b.h5'),
             os.path.join(url_models, 'model_b.h5')
         )
         model_type = 'weights'
         return model_type
     except Exception:
         tf.keras.utils.get_file(
-            os.path.join(model_path, 'model.h5'),
+            model_path.joinpath('model.h5'),
             os.path.join(url_models, 'model.h5')
         )
         tf.keras.utils.get_file(
-            os.path.join(model_path, 'model_background.h5'),
+            model_path.joinpath('model_background.h5'),
             os.path.join(url_models, 'model_background.h5')
         )
         return 'model'
 
 
-def fetch_state_dict(url: str, filepath: str, save_state_dict: bool = True) -> Tuple[dict, dict]:
+def fetch_state_dict(url: str, filepath: Union[str, os.PathLike],
+                     save_state_dict: bool = True) -> Tuple[dict, dict]:
     """
     Fetch the metadata and state/hyperparameter values of pre-trained detectors.
 
@@ -400,20 +406,44 @@ def fetch_state_dict(url: str, filepath: str, save_state_dict: bool = True) -> T
     -------
     Detector metadata and state.
     """
-    # fetch and save metadata and state dict
-    path_meta = os.path.join(url, 'meta.pickle')
-    meta = cp.load(urlopen(path_meta))
-    path_state = os.path.join(url, meta['name'] + '.pickle')
-    state_dict = cp.load(urlopen(path_state))
+    # Check if metadata stored as dill or pickle
+    try:
+        url_meta = os.path.join(url, 'meta.dill')
+        resp = requests.get(url_meta, timeout=2)
+        resp.raise_for_status()
+        suffix = '.dill'
+    except RequestException:
+        try:
+            url_meta = os.path.join(url, 'meta.pickle')
+            resp = requests.get(url_meta, timeout=2)
+            resp.raise_for_status()
+            suffix = '.pickle'
+        except RequestException:
+            logger.exception('Timed out while searching for meta.dill or meta.pickle files at %s.', url)
+            raise
+
+    # Load metadata and state_dict
+    meta = dill.load(BytesIO(resp.content))
+    try:
+        url_state = os.path.join(url, meta['name'] + suffix)
+        resp = requests.get(url_state)
+        resp.raise_for_status()
+    except RequestException:
+        logger.exception('Timed out while searching for corresponding state file at %s.', url)
+        raise
+    state_dict = dill.load(BytesIO(resp.content))
+
+    # Save state
     if save_state_dict:
-        with open(os.path.join(filepath, 'meta.pickle'), 'wb') as f:
-            pickle.dump(meta, f)
-        with open(os.path.join(filepath, meta['name'] + '.pickle'), 'wb') as f:
-            pickle.dump(state_dict, f)
+        filepath = Path(filepath)
+        with open(filepath.joinpath('meta.dill'), 'wb') as f:
+            dill.dump(meta, f)
+        with open(filepath.joinpath(meta['name'] + '.dill'), 'wb') as f:
+            dill.dump(state_dict, f)
     return meta, state_dict
 
 
-def fetch_detector(filepath: str,
+def fetch_detector(filepath: Union[str, os.PathLike],
                    detector_type: str,
                    dataset: str,
                    detector_name: str,
@@ -439,11 +469,12 @@ def fetch_detector(filepath: str,
     -------
     Initialised pre-trained detector.
     """
-    # create url of detector
-    filepath = os.path.join(filepath, detector_name)
-    if not os.path.isdir(filepath):
+    # create path (if needed)
+    filepath = Path(filepath)
+    if not filepath.is_dir():
+        filepath.mkdir(parents=True, exist_ok=True)
         logger.warning('Directory {} does not exist and is now created.'.format(filepath))
-        os.mkdir(filepath)
+    # create url of detector
     url = 'https://storage.googleapis.com/seldon-models/alibi-detect/'
     if detector_type == 'adversarial':
         url = os.path.join(url, 'ad', dataset, model, detector_name)
