@@ -1,15 +1,17 @@
-from .distance import mmd2, mmd2_from_kernel_matrix
+from .distance import mmd2, mmd2_from_kernel_matrix, batch_compute_kernel_matrix
 from .distance import relative_euclidean_distance, squared_pairwise_distance, permed_lsdds
-from .kernels import GaussianRBF
+from .kernels import GaussianRBF, DeepKernel
 from .prediction import predict_batch, predict_batch_transformer
 from .misc import zero_diag, quantile, subset_matrix
 
 __all__ = [
+    "batch_compute_kernel_matrix",
     "mmd2",
     "mmd2_from_kernel_matrix",
     "relative_euclidean_distance",
     "squared_pairwise_distance",
     "GaussianRBF",
+    "DeepKernel",
     "permed_lsdds",
     "predict_batch",
     "predict_batch_transformer",
